 
// Copyright (c) 1995 Jeff Weisberg
// University of Rochester
//      Dept. of Electrical Engineering
//      Dept. of Neurology, C.E.P.
//
// see the file "License"
 
#ifdef RCSID
static const char *const rcsid
= "@(#)$Id$";
#endif

// Optimization: None

#include <rdb.h>
#include <info.h>
#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>

#include <signa-consts.h>
#include <gemr_imgf.h>

#include <string.h>
#include <dirent.h>
#include <time.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fstream.h>
#include <unistd.h>

#define direct dirent

Patient *patients_l = 0;
Patient **patients_a = 0;
int num_patients = 0;

char *rawdir;


int cmp_pat(Patient **a, Patient **b){
	return strcmp( (**a).name, (**b).name);
}

int cmp_img(Image **a, Image **b){
	return strcmp( (**a).name, (**b).name);
}

extern "C" void update_db(void){
	int i;
	
	for(i=0; i<num_patients; i++)
		delete patients_a[i];
	delete [] patients_a;
	patients_a = 0;
	patients_l = 0;
	create_db( rawdir );
	
}

void create_db(char *raw){
	DIR *dot, *top;
	Patient *newone, *prev=0;
	struct direct *dotent, *topent;
	int total=0;

	rawdir = raw;
	
	chdir( rawdir );
	
	if ( !(dot=opendir("."))){
		cerr << "Could not open current directory!!!" << endl;
		exit(-1);
	}

	while ( (dotent=readdir(dot)) ){ /* for@ toplevel dir */
		if ( dotent->d_name[0] =='.' ) continue;
		if (!(top=opendir(dotent->d_name))) continue; 		/* may just be a file */
		while ( (topent=readdir(top))){ 			/* for@ patient in here */
			if ( topent->d_name[0] == '.' ) continue;
			if (!isdigit(topent->d_name[0])) continue; 	/* make sure PID is numeric */
			newone = new Patient;
			
			sprintf(newone->dir, "%s/%s", dotent->d_name, topent->d_name);
			strncpy(newone->pid, topent->d_name, STR_LEN);
			newone->num = atoi( newone->pid );
			if( newone->learn()){
				total ++;
				if(!patients_l) patients_l = newone;
				else prev->next = newone;
				prev = newone;
			}else{
				delete newone;
				
			}
			chdir(rawdir);
		}
		closedir(top);
	}
	closedir(dot);
        /* we now have ll of patient IDs and their assoc dir. */

	Patient *curr_p = patients_l;
	int i = 0;
	
	patients_a = new Patient* [total+1];
	num_patients = total;
	
	curr_p = patients_l;
	while (curr_p){
		patients_a[i++] = curr_p;
		curr_p = curr_p->next;
	}

}

int Patient::learn(void){
	// gather data on this patient

	DIR	*patdir, *studydir;
	Exam	*curr_s, *head_s = 0, *prev_s;
	struct direct *ent;
	
	chdir(rawdir);
	patdir = opendir(dir);
	if (chdir(dir)==-1) return 0;

	while ( (patdir)&&(ent=readdir(patdir)) ){	// for@ study
		if ( ent->d_name[0] == '.') continue;
		if ( !(studydir=opendir(ent->d_name)) ) continue; // not a dir
		if(chdir(ent->d_name)==-1) continue;	// not a dir?
		curr_s = new Exam;
		
		curr_s->back = this;
		strncpy(curr_s->name, ent->d_name, STR_LEN);
		sprintf(curr_s->dir, "%s/%s", dir, ent->d_name);
		curr_s->num = atoi( curr_s->name );
		
		if(curr_s->learn()){			// gather and report on data here
			num_exams++;
			flag = 1;
			if(!head_s){
				head_s=curr_s;
				prev_s=curr_s;
			}else{
				prev_s->next = curr_s;
				prev_s = curr_s;
			}
		}else{
			delete curr_s;
		}
		closedir(studydir);
		chdir(rawdir);		
		chdir(dir);		
	}

	closedir(patdir);
	chdir("..");

	// build study cache and attach it
	exams = new Exam* [num_exams];
		
	curr_s = head_s;
	int foo=0;
	while (curr_s){
		exams[foo++] = curr_s;
		curr_s = curr_s->next;
	}

	return flag;
}


int Exam::learn(void){
	// gather data on this study

	DIR	*studir, *serdir;
	Series	*curr_s, *head_s = 0, *prev_s;
	struct direct *ent;
	
	chdir(rawdir);
	studir = opendir(dir);
	if (chdir(dir)==-1) return 0;	// not a dir

	while ( (studir)&&(ent=readdir(studir)) ){	// for@ sseries
		if ( ent->d_name[0] == '.') continue;
		if ( !(serdir=opendir(ent->d_name)) ) continue;
		chdir(ent->d_name);
		curr_s = new Series;
		curr_s->back = this;
		
		strncpy(curr_s->name, ent->d_name, STR_LEN);
		sprintf(curr_s->dir,  "%s/%s", dir, ent->d_name);
		curr_s->num = atoi( curr_s->name );
		
		if(curr_s->learn()){			// gather and report on data here
			num_series++;
			flag = 1;
			if(!head_s){
				head_s=curr_s;
				prev_s=curr_s;
			}else{
				prev_s->next = curr_s;
				prev_s = curr_s;
			}
		}else{
			delete curr_s;
		}
		closedir(serdir);
		chdir(rawdir);	
		chdir(dir);	

	}

	closedir(studir);
	chdir("..");

	series = new Series* [num_series];
		
	curr_s = head_s;
	int foo=0;
	while (curr_s){
		series[foo++] = curr_s;
		curr_s = curr_s->next;
	}

	return flag;
}

int Series::learn(void){
	// gather data on this series

	DIR	*serdir;
	Image	*curr_i, *head_i = 0, *prev_i;
	struct direct *ent;
	
	chdir(rawdir);
	serdir = opendir(dir);
	if (chdir(dir)==-1) return 0;	// not a dir

	while ( (serdir)&&(ent=readdir(serdir)) ){	// for@ image
		if ( ent->d_name[0] != 'I') continue;
		// assume images are named in the form: I.nnn[.Z | .gz]
		if ( (!isdigit(ent->d_name[2]))||(!isdigit(ent->d_name[3]))) continue; /*well, huh? is it?*/
		curr_i = new Image;
		curr_i->back = this;

		strncpy(curr_i->name,ent->d_name, STR_LEN);
		sprintf(curr_i->dir, "%s/%s", dir, ent->d_name);
		sscanf(ent->d_name, "I.%d", &(curr_i->num) );

		if(curr_i->learn()){			// gather and report on data here
			num_images++;
			flag = 1;
			if(!head_i){
				head_i=curr_i;
				prev_i=curr_i;
			}else{
				prev_i->next = curr_i;
				prev_i = curr_i;
			}
		}else{
			delete curr_i;
		}
	}

	closedir(serdir);
	chdir("..");

	images = new Image* [num_images];
		
	curr_i = head_i;
	int foo=0;
	while (curr_i){
		images[foo++] = curr_i;	
		curr_i = curr_i->next;		
		
	}
	qsort(images, num_images, sizeof( Image* ), (int(*)(const void*, const void*))cmp_img);

	return flag;
}

int Image::learn(void){
	// slurp data from image

	FILE *fp;
	Series *c_ser = back;
	Exam *c_stu = c_ser->back;
	Patient *c_pat = c_stu->back;
	struct GEMR_Header hdr;
	int pipep=0, c;

	if(!c_ser->flag){
		char *cmd;
		
		c = name[strlen(name)-1];
		switch(c){
		case 'Z':
			cmd = "uncompress -c 2>/dev/null";
			pipep = 1;
			break;
		case 'z':
			cmd = "gunzip -c 2>/dev/null";
			pipep = 1;
			break;
		}

		if(pipep){
			char buf[8192];
			sprintf(buf, "%s %s", cmd, name);
			fp = popen(buf, "r");
		}else{
			fp = fopen(name, "r");
		}
		if( !fp ){
				// error
			return 0;
		}

		hdr.read_header(fp);
		/* we got one, milk as much data as we can from it*/

		strncpy(c_ser->desc, hdr.get_str_se_desc(), STR_LEN);
		c_ser->desc[59] = 0;
		strncpy(c_ser->date,  hdr.get_str_se_date(), STR_LEN);
		c_ser->date[15] = 0;
		strncpy(c_ser->proto, hdr.get_str_se_protocol(), STR_LEN);
		c_ser->proto[31] = 0;

		if(!c_stu->flag){
			strncpy(c_stu->date,  hdr.get_str_st_date(), STR_LEN);
			c_stu->date[15] = 0;
			strncpy(c_stu->desc, hdr.get_str_st_desc(), STR_LEN);
			c_stu->desc[59] = 0;
			
			if(!c_pat->flag){
				strncpy(c_pat->name, hdr.get_str_pt_name(), STR_LEN);
				c_pat->name[31] = 0;
				strncpy(c_pat->hosp, hdr.get_str_hosp(), STR_LEN);
				c_pat->hosp[31] = 0;
				c_pat->sex = hdr.get_int_pt_sex();
				c_pat->age = hdr.get_int_pt_age();
				c_pat->weight = hdr.get_int_pt_weight();
			}
		}
		if(pipep)
			pclose(fp);
		else
			fclose(fp);
	}
	return 1;
}


Patient *find(int n){
	int i;

	for(i=0;i<num_patients;i++)
		if(n == patients_a[i]->num) return patients_a[i];

	return 0;
}

Exam* Patient::find(int n) const {
	int i;

	if( !this) return 0;
	
	for(i=0;i<num_exams;i++)
		if(n == exams[i]->num) return exams[i];

	return 0;
}

Series* Exam::find(int n) const {
	int i;

	if( !this) return 0;
	
	for(i=0;i<num_series;i++)
		if(n == series[i]->num) return series[i];

	return 0;
}

Image *Series::find(int n) const {
	int i;

	if( !this) return 0;
	
	for(i=0;i<num_images;i++)
		if(images[i]->num == n) return images[i];

	return 0;
}




	
		
