 
// Copyright (c) 1994 Jeff Weisberg
// University of Rochester
//      Dept. of Electrical Engineering
//      Dept. of Neurology, C.E.P.
//
// see the file "License"
 
#ifdef RCSID
static const char *const rcsid
= "@(#)$Id: file.cc,v 1.24 95/01/21 05:11:32 weisberg Exp Locker: weisberg $";
#endif

// Optimization: Off

#include <rdb.h>
#include <info.h>
#include <stdio.h>
#include <fstream.h>

#include <ctype.h>
#include <string.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>

#include <signa-consts.h>	// 4.x header defn
#include <gemr_imgf.h>		// 5.x header defn

float dg2sun(int hi, int lo);
extern struct sockaddr_in client_addr;
extern char *rawdir;

union ge_xxx_4 {
	char	c[4];
	int	i;
	float	f;
};
union ge_xxx_2 {
	char	c[2];
	short	s;
};


// read the header, handles 4x and 5x headers
int GEMR_Header::read_header(FILE *fp){
	int s, l;
	
	// grab a bit off the front to determine file type
	s = fread(u.raw, 128, 1, fp);
	
	if( u.ge5x.img_magic == IMGF_MAGIC ){
		// 5x format
		tag = 5;

		if( u.ge5x.img_version != 3){
			return 0; // error
		}

		l = u.ge5x.img_hdr_length;
		if( l > sizeof(u)){
			return 0; // error
		}
		s = fread(u.raw + 128, 1, u.ge5x.img_hdr_length - 128, fp);
		p.im = (void*)u.raw;
		p.su = u.ge5x.img_l_suite  ? (void*) (u.raw + u.ge5x.img_p_suite ) : 0;
		p.ex = u.ge5x.img_l_exam   ? (void*) (u.raw + u.ge5x.img_p_exam  ) : 0;
		p.se = u.ge5x.img_l_series ? (void*) (u.raw + u.ge5x.img_p_series) : 0;
		p.mr = u.ge5x.img_l_image  ? (void*) (u.raw + u.ge5x.img_p_image ) : 0;
		p.up = u.ge5x.img_l_unpack ? (void*) (u.raw + u.ge5x.img_p_unpack) : 0;

	}else{
		// 4x format
		tag = 4;
		s = fread(u.raw + 128, 1, 256*28*sizeof(short) - 128, fp);
	}
	return tag;
}

static void get_xxx4(union ge_xxx_4 *u, char* p){
	u->c[0] = p[0];
	u->c[1] = p[1];
	u->c[2] = p[2];
	u->c[3] = p[3];
}
static void get_xxx2(union ge_xxx_2 *u, char* p){
	u->c[0] = p[0];
	u->c[1] = p[1];
}
static int get_int(char *p){
	union ge_xxx_4 u;
	get_xxx4(&u, p);
	return u.i;
}
static float get_float(char *p){
	union ge_xxx_4 u;
	get_xxx4(&u, p);
	return u.f;
}
static int get_short(char *p){
	union ge_xxx_2 u;
	get_xxx2(&u, p);
	return u.s;
}

int GEMR_Header::get_int_pt_age(void){
	if(tag==4){
		return atoi( u.ge4x.c + STHDR_START+AGE );
	}else{
		return get_short( (char*)p.ex + _EX_patage );
	}
}
	
int GEMR_Header::get_int_pt_sex(void){
	if(tag==4){
		return u.ge4x.c[STHDR_START+SEX];
	}else{
		return get_short( (char*)p.ex + _EX_patsex );
	}
}

int GEMR_Header::get_int_pt_weight(void){
	if(tag==4){
		return atoi( u.ge4x.c + STHDR_START+WEIGHT );
	}else{
		return get_short( (char*)p.ex + _EX_patweight );
	}
}

int GEMR_Header::get_int_mr_xdim(void){
	if(tag==4){
		return u.ge4x.s[(IMHDR_START+IMXPIXDIM)/2];
	}else{
		return get_short( (char*)p.mr + _MR_imatrix_X );
	}
}

int GEMR_Header::get_int_mr_ydim(void){
	if(tag==4){
		return u.ge4x.s[(IMHDR_START+IMXPIXDIM)/2];
	}else{
		return get_short( (char*)p.mr + _MR_imatrix_Y );
	}
}

char * GEMR_Header::get_str_se_protocol(void){
	if(tag==4){
		return u.ge4x.c + SEHDR_START + SERPROTO;
	}else{
		return (char*)p.se + _SE_prtcl;
	}
}

char * GEMR_Header::get_str_se_date(void){
	if(tag==4){
		return u.ge4x.c + SEHDR_START+SERDATE;
	}else{
		int t;
		struct tm *stm;
		static char buf[64];
		
		t = get_int( (char*)p.se + _SE_se_datetime );
		stm = localtime((time_t*) &t);
		strftime(buf, sizeof(buf), "%d-%h-%Y", stm);
		return buf;
	}
}

char * GEMR_Header::get_str_st_date(void){
	if(tag==4){
		return u.ge4x.c + STHDR_START+STDATE;
	}else{
		int t;
		struct tm *stm;
		static char buf[64];
		
		t = get_int( (char*)p.ex + _EX_ex_datetime );
		stm = localtime((time_t*) &t);
		strftime(buf, sizeof(buf), "%d-%h-%Y", stm);
		return buf;
	}
}

char * GEMR_Header::get_str_se_desc(void){
	if(tag==4){
		return u.ge4x.c + SEHDR_START+SERDESCR;
	}else{
		return (char*)p.se + _SE_se_desc;
	}
}
char * GEMR_Header::get_str_st_desc(void){
	if(tag==4){
		return u.ge4x.c + STHDR_START+STDESCR;
	}else{
		return (char*)p.ex + _EX_ex_desc;
	}
}
	
char * GEMR_Header::get_str_pt_name(void){
	if(tag==4){
		return u.ge4x.c + NAME_START+STHDR_START;
	}else{
		return (char*)p.ex + _EX_patname;
	}
}
char * GEMR_Header::get_str_hosp(void){
	if(tag==4){
		return u.ge4x.c + HOSP+STHDR_START;
	}else{
		return (char*)p.ex + _EX_hospname;
	}
}
float GEMR_Header::get_float_mr_thick(void){
	if(tag==4){
		return dg2sun( (int)u.ge4x.s[(IMHDR_START+IMTHICK)/2], 
			       (int)u.ge4x.s[(IMHDR_START+IMTHICK)/2+1] );
	}else{
		return get_float( (char*)p.mr + _MR_slthick );
	}
}

float GEMR_Header::get_float_mr_fov(void){
	if(tag==4){
		return dg2sun( (int)u.ge4x.s[(SEHDR_START+SERFOV)/2], 
			       (int)u.ge4x.s[(SEHDR_START+SERFOV)/2+1] );
	}else{
		return get_float( (char*)p.mr + _MR_dfov );
	}
}
float GEMR_Header::get_float_mr_psize(void){
	if(tag==4){
		return dg2sun( (int)u.ge4x.s[(IMHDR_START+IMPIX)/2], 
			       (int)u.ge4x.s[(IMHDR_START+IMPIX)/2+1] );
	}else{
		return get_float( (char*)p.mr + _MR_pixsize_X );
	}
}


void data(FILE *in, FILE *out, char *txt){
	int i=0,n=0;
	char ntxt[256];
	int tl=strlen(txt);
	int pn, en, sn, inm, port;
	Image *im;
	char *cmd;
	int pipep=0;
	FILE *fp;
	
	while( n<tl && isspace(txt[n]))
		n++;
	while( n<tl && isdigit(txt[n]))
		ntxt[i++] = txt[n++];
	ntxt[i] = 0;
	pn = atoi(ntxt);

	i = 0;
	while( n<tl && isspace(txt[n]))
		n++;
	while( n<tl && isdigit(txt[n]))
		ntxt[i++] = txt[n++];
	ntxt[i] = 0;
	en = atoi(ntxt);

	i = 0;
	while( n<tl && isspace(txt[n]))
		n++;
	while( n<tl && isdigit(txt[n]))
		ntxt[i++] = txt[n++];
	ntxt[i] = 0;
	sn = atoi(ntxt);

	i = 0;
	while( n<tl && isspace(txt[n]))
		n++;
	while( n<tl && isdigit(txt[n]))
		ntxt[i++] = txt[n++];
	ntxt[i] = 0;
	inm = atoi(ntxt);

	i = 0;
	while( n<tl && isspace(txt[n]))
		n++;
	while( n<tl && isdigit(txt[n]))
		ntxt[i++] = txt[n++];
	ntxt[i] = 0;
	port = atoi(ntxt);


	im = find(pn)->find(en)->find(sn)->find(inm);
	if(! im){
		fprintf(out, "ERROR not found\n");
		fflush(out);
		return;
	}

	switch( im->name[strlen(im->name)-1] ){
	case 'Z':
		cmd = "uncompress -c 2>/dev/null";
		pipep = 1;
		break;
	case 'z':
		cmd = "gunzip -c 2>/dev/null";
		pipep = 1;
		break;
	}

	chdir( rawdir );
	if(pipep){
		char buf[8192];
		sprintf(buf, "%s %s", cmd, im->dir);
		fp = popen(buf, "r");
	}else{
		fp = fopen(im->dir, "r");
	}
	if( !fp ){
		fprintf(out, "ERROR could not access image data\n");
		fflush(out);
		return;
	}

	// create data connection
	int fd, ns;
	struct sockaddr_in sain;
	
	fd = socket(AF_INET, SOCK_STREAM, 0);

	// connect to client at specified port
	ns = sizeof(sain);
	sain = client_addr;
	sain.sin_port = htons( port );

	if( connect(fd, (sockaddr*)&sain, sizeof(sain)) ==-1){
		fprintf(out, "ERROR connect failed\n");
		fflush(out);
		return;
	}

	ns = sizeof(sain);
	getsockname(fd, (sockaddr*)&sain, &ns);
	fprintf(out, "OK %d\n", ntohs(sain.sin_port));
	fflush(out);

	// cat file from fp -> fd

	char buffer[1024*1024];
	int nr;
	while(! feof(fp)){
		nr = fread(buffer, 1, sizeof(buffer), fp);
		write(fd, buffer, nr);
	}

	close( fd );
	if(pipep)
		pclose(fp);
	else
		fclose(fp);
	
	fflush(out);

}

