
// Copyright (c) 1995 Jeff Weisberg
// University of Rochester
//      Dept. of Electrical Engineering
//      Dept. of Neurology, C.E.P.
//
// see the file "License"
 
#ifdef RCSID
static const char *const rcsid
= "@(#)$Id$";
#endif

// Optimization: None


#include <rdb.h>
#include <info.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <signal.h>
#include <netinet/in.h>
#include <netdb.h>
 
#include <stdio.h>
#include <string.h>
#include <unistd.h>

void server_deal(int fd);

extern void auth(FILE*, FILE*, char*);
extern void data(FILE*, FILE*, char*);
extern void dele(FILE*);
extern void arcv(FILE*);

extern char *rawdir;
extern Patient **patients_a;
extern int num_patients;

FILE *cl_in, *cl_out;
struct sockaddr_in client_addr;
int authorized = 0;

struct Tok {
	char *txt;
	int val;
};

struct Tok tox[] = {
	{ "HELO", TOK_HELO },
	{ "AUTH", TOK_AUTH },
	{ "LIST", TOK_LIST },
	{ "INFO", TOK_INFO },
	{ "MARK", TOK_MARK },
	{ "DELE", TOK_DELE },
	{ "ARCV", TOK_ARCV },
	{ "DATA", TOK_DATA },
	{ "UPDB", TOK_UPDB },
	{ "HELP", TOK_HELP },
	{ "QUIT", TOK_QUIT },
	{ 0, 0 }
};

int tokval(char *txt){
	int i;

	for(i=0; tox[i].txt; i++){
		if(! strncmp( tox[i].txt, txt, 4))
			return tox[i].val;
	}
	return 0;
}
	

int create_server(void){
	int fd;
	int cmd;
	struct sockaddr_in name;
	struct servent *sp;
	
	fd = socket(AF_INET, SOCK_STREAM, 0);
	if( fd == -1){
		perror("create server: socket failed");
		exit(-1);
	}

	cmd = 1;
	setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, (const char*)&cmd, sizeof(cmd));
	setsockopt(fd, SOL_SOCKET, SO_DEBUG,     (const char*)&cmd, sizeof(cmd));
	setsockopt(fd, SOL_SOCKET, SO_KEEPALIVE, (const char*)&cmd, sizeof(cmd));

	name.sin_addr.s_addr = INADDR_ANY;
	sp = getservbyname( "mri-rdb", 0);
	name.sin_port = sp ? sp->s_port : htons( MRI_RDB_PORT );
	
	if( bind(fd, (sockaddr*)&name, sizeof(name))){
		perror("create server: bind failed");
		exit( -1 );
	}
	
	listen(fd, 5);

	return fd;
}


void server_loop(int sk){
	int fd;
	struct sockaddr_in name;
	int nl;
	
	while(1){
		nl = sizeof(name);
		fd = accept(sk, (sockaddr*)&name, &nl);
		client_addr = name;
		// block until client connects

		if( fd == -1)
			// error ?
			continue;

#ifndef DEBUG
		// fork child to handle it
		if( fork()){
			// parent close connection and continue
			close( fd );
		}else{
			// child
			setsid();	// keep connection alive if parent dies
			close(sk);	// to allow parent to play with it
			server_deal(fd);
			close(fd);
			_exit(0);
		}
#else
		close(sk);
		server_deal(fd);
		close(fd);
#endif		
	}
}


void server_deal(int fd){
	char buf[8192];
	char bufn[256];
	int i,n;

	authorized = 0;
	cl_in = fdopen(fd, "r");
	cl_out= fdopen(fd, "w");

	fprintf(cl_out, "HELO \n");
	fflush(cl_out);
	
	fgets(buf, sizeof(buf), cl_in);
	if( strncmp(buf, "HELO", 4))
		return;

	fprintf(cl_out, "OK\n");
	fflush(cl_out);

	while(1){
		
		fgets(buf, sizeof(buf), cl_in);
		if( feof( cl_in))
			break;
		
		switch( tokval( buf )){

		case TOK_QUIT:
			fprintf(cl_out, "OK\n");
			fflush(cl_out);
			fclose(cl_in);
			fclose(cl_out);
			close(fd);
			_exit(0);
			break;
			
		case TOK_NULL:
			fprintf(cl_out, "ERROR unrecognized command\n");
			fflush(cl_out);
			break;

		case TOK_HELO:
			fprintf(cl_out, "OK (still here)\n");
			fflush(cl_out);
			break;

		case TOK_LIST:
			list( cl_out, buf+4 );
			break;
			
		case TOK_INFO:
			info( cl_out, buf+4 );
			break;

		case TOK_MARK:
			mark( cl_out, buf+4 );
			break;

		case TOK_HELP:
			fprintf(cl_out, "ERROR help yourself\n");
			fflush(cl_out);
			break;
			
		case TOK_UPDB:
			if(authorized){
				// this process only
				// is this really of any use?
				kill(getpid() ,SIGUSR1);
				fprintf(cl_out, "OK\n");
			}else
				fprintf(cl_out, "ERROR not authorized\n");
			fflush(cl_out);
			break;
			
		case TOK_AUTH:
			auth(cl_in, cl_out, buf+4);
			break;

		case TOK_DATA:
			if(authorized)
				data(cl_in, cl_out, buf+4);
			else{
				fprintf(cl_out, "ERROR not authorized\n");
				fflush(cl_out);
			}
			break;
			
		case TOK_DELE:
			dele(cl_out);
			break;
			
		case TOK_ARCV:
			arcv(cl_out);
			break;
			
		default:
			fprintf(cl_out, "ERROR unsupported feature\n");
			fflush(cl_out);
			break;
		}
		fflush(cl_out);
	}
	fclose( cl_in );
	fclose( cl_out);
}

